import os
import ConfigParser

import click

from .templates import setup_py, setup_cfg, gitignore, readme, mit
from .constants import RC_FILE, CONTEXT_SETTINGS


def _bootstrap():
    author = click.prompt("What is your name?")
    email = click.prompt("What is your preferred email?")
    license = click.prompt("What license do you prefer?")

    return author, email, license


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    pass


@click.command(context_settings=CONTEXT_SETTINGS)
@click.option('--project', help='Name of your new project.')
@click.option('--description', help='Description of your project.')
@click.argument('path', default='.')
def create(project, description, path):
    # make dirs
    if not os.path.exists(path):
        os.makedirs(path)

    # establish namespace
    os.makedirs(os.path.join(path, 'src', project))
    with open(os.path.join(path, 'src', project, '__init__.py'), 'w') as fh:
        fh.write('# This is a package.')

    # get defaults
    if os.path.exists(RC_FILE):
        config = ConfigParser.RawConfigParser()
        config.read(RC_FILE)
        author = config.get('setuptize', 'author')
        email = config.get('setuptize', 'email')
        license = config.get('setuptize', 'license')
    else:
        author, email, license = _bootstrap()

    # create setup.py
    with open(os.path.join(path, 'setup.py'), 'w') as fh:
        fh.write(
            setup_py.safe_substitute(
                project_name=project,
                description=description,
                author=author,
                email=email,
                license=license,
                requirements=['pex', 'setuptools'],
            )
        )

    if license.lower() == 'mit':
        # create license
        with open(os.path.join(path, 'LICENSE'), 'w') as fh:
            fh.write(mit.substitute(name=author))

    # create readme
    with open(os.path.join(path, 'README.md'), 'w') as fh:
        fh.write(readme.substitute(project_name=project))

    # create setup.cfg
    with open(os.path.join(path, 'setup.cfg'), 'w') as fh:
        fh.write(setup_cfg)

    # create gitignore
    with open(os.path.join(path, '.gitignore'), 'w') as fh:
        fh.write(gitignore)


@click.command(context_settings=CONTEXT_SETTINGS)
def bootstrap():
    author, email, license = _bootstrap()

    overwrite = click.confirm(
        "Warning, this will overwrite your ~/.setuptize config! Proceed? [Yn]",
        abort=True
    )

    if not overwrite:
        raise SystemExit(0)

    with open(RC_FILE, 'w') as fh:
        config = ConfigParser.RawConfigParser()
        config.add_section('setuptize')
        config.set('setuptize', 'author', author)
        config.set('setuptize', 'email', email)
        config.set('setuptize', 'license', license)
        config.write(fh)


cli.add_command(create)
cli.add_command(bootstrap)
